-- sql/objects-create.sql
--
-- PL/SQL that makes various portal tables into acs objects.
-- It's stored here to keep the data model clear.
--
-- Ian Baker <ibaker@arsdigita.com>
-- $Id: objects-create.sql,v 1.1 2002/07/09 17:35:10 rmello Exp $
--


-- First, the portal type.
begin  
  acs_object_type.create_type ( 
    supertype     => 'acs_object', 
    object_type   => 'portal', 
    pretty_name   => 'Portal', 
    pretty_plural => 'Portals', 
    table_name    => 'PORTALS', 
    id_column     => 'PORTAL_ID',
    package_name  => 'portal'
  ); 
end;
/
show errors;

-- and its attributes, 'name', and 'cache_time'
declare 
 attr_id acs_attributes.attribute_id%TYPE; 
begin
  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal', 
    attribute_name => 'NAME', 
    pretty_name    => 'Name', 
    pretty_plural  => 'Names', 
    datatype       => 'string' 
  );
end; 
/ 
show errors;


-- portal_templates, now.
begin  
  acs_object_type.create_type (
    supertype     => 'acs_object',
    object_type   => 'portal_template',
    pretty_name   => 'Portal Template',
    pretty_plural => 'Portal Templates',
    table_name    => 'PORTAL_TEMPLATES',
    id_column     => 'TEMPLATE_ID',
    package_name  => 'portal_template'
  );
end;
/
show errors;

-- and its attributes
declare 
 attr_id acs_attributes.attribute_id%TYPE; 
begin
  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_template', 
    attribute_name => 'NAME', 
    pretty_name    => 'Name', 
    pretty_plural  => 'Names', 
    datatype       => 'string' 
  ); 

  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_template', 
    attribute_name => 'DESCRIPTION', 
    pretty_name    => 'Description', 
    pretty_plural  => 'Descriptions', 
    datatype       => 'string' 
  ); 

  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_template', 
    attribute_name => 'TYPE', 
    pretty_name    => 'Type', 
    pretty_plural  => 'Types', 
    datatype       => 'string' 
  ); 

  attr_id := acs_attribute.create_attribute (
    object_type    => 'portal_template',
    attribute_name => 'FILENAME',
    pretty_name    => 'Filename',
    pretty_plural  => 'Filenames',
    datatype       => 'string'
  ); 

  attr_id := acs_attribute.create_attribute (
    object_type    => 'portal_template',
    attribute_name => 'SUPPORTED_REGIONS',
    pretty_name    => 'Supported Regions',
    pretty_plural  => 'Supported Regions',
    datatype       => 'string'
  ); 

  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_template', 
    attribute_name => 'EXAMPLE',
    pretty_name    => 'Example',
    pretty_plural  => 'Examples',
    datatype       => 'string' 
  );
end; 
/ 
show errors;


-- element configurations,
-- which have no attributes of their own.
begin  
  acs_object_type.create_type (
    supertype     => 'acs_object',
    object_type   => 'portal_element_config',
    pretty_name   => 'Portal Element Configuration',
    pretty_plural => 'Portal Element Configurations',
    table_name    => 'PORTAL_ELEMENT_CONFIGS',
    id_column     => 'CONFIG_ID',
    package_name  => 'portal_config'
  );
end;
/
show errors;


-- datasources
begin  
  acs_object_type.create_type ( 
    supertype     => 'acs_object',
    object_type   => 'portal_datasource',
    pretty_name   => 'Portal Data Source',
    pretty_plural => 'Portal Data Sources',
    table_name    => 'PORTAL_DATASOURCES',
    id_column     => 'DATASOURCE_ID',
    package_name  => 'portal_datasource'
  ); 
end;
/
show errors;

-- datasource attributes
declare 
 attr_id acs_attributes.attribute_id%TYPE; 
begin
  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_datasource', 
    attribute_name => 'NAME', 
    pretty_name    => 'Name', 
    pretty_plural  => 'Names', 
    datatype       => 'string' 
  ); 

  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_datasource', 
    attribute_name => 'DESCRIPTION', 
    pretty_name    => 'Description', 
    pretty_plural  => 'Descriptions', 
    datatype       => 'string' 
  ); 

  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_datasource', 
    attribute_name => 'CONTENT', 
    pretty_name    => 'Content', 
    pretty_plural  => 'Contents', 
    datatype       => 'string' 
  ); 
end; 
/ 
show errors;


-- Finally, elements.
begin  
  acs_object_type.create_type ( 
    supertype     => 'acs_object', 
    object_type   => 'portal_element',
    pretty_name   => 'Portal Element',
    pretty_plural => 'Portal Elements',
    table_name    => 'PORTAL_ELEMENTS', 
    id_column     => 'ELEMENT_ID',
    package_name  => 'portal_element'
  ); 
end;
/
show errors;

-- and attributes for the element object...
declare 
 attr_id acs_attributes.attribute_id%TYPE; 
begin
  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_element', 
    attribute_name => 'NAME', 
    pretty_name    => 'Name', 
    pretty_plural  => 'Names', 
    datatype       => 'string' 
  ); 

  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_element',
    attribute_name => 'DESCRIPTION', 
    pretty_name    => 'Description', 
    pretty_plural  => 'Descriptions', 
    datatype       => 'string' 
  );

  attr_id := acs_attribute.create_attribute ( 
    object_type    => 'portal_element',
    attribute_name => 'EXPORTABLE_P', 
    pretty_name    => 'Exportable', 
    pretty_plural  => 'Exportable', 
    datatype       => 'string' 
  );
end; 
/ 
show errors;
