# www/element-config-2.tcl

ad_page_contract {
    Update local element configuration.

    @author Ian Baker (ibaker@arsdigita.com)
    @creation-date 1/14/2000
    @cvs-id $Id: element-config-2.tcl,v 1.1 2002/07/09 17:35:10 rmello Exp $
} {
    element_id:naturalnum,permission(write)
}

set package_id [ad_conn package_id]
set user_id [ad_conn user_id]

ad_require_permission $package_id admin

# ...the hard way
set form [ns_getform]

db_transaction {

    db_1row get_element_info \
	"select
           e.config_id,
           d.default_config_id,
           decode(e.config_id, d.default_config_id, 1, 0) as default_config_p,
           decode(d.configurable_p, 't', 1, 'f', 0) as configurable_p
         from portal_datasources d, portal_elements e
         where e.datasource_id = d.datasource_id and e.element_id = :element_id"

    if { ! $configurable_p } {
	ad_return_complaint 1 "This element doesn't accept a configuration."
	return
    }

    lappend default_keys {}

    db_foreach get_default_keys \
	"select key from portal_element_parameters where config_id = :default_config_id" {
	    lappend default_keys $key
	}

    if { $default_config_p } {
	# this element is using the default configuration.
	set config_id [db_nextval acs_object_id_seq]

	db_dml insert_new_config \
	    "declare
               v_config_id portal_element_configs.config_id%TYPE;
             begin
               v_config_id := portal_element_config.new(config_id => :config_id);
             end;"
	db_dml update_element \
	    "update portal_elements set config_id = :config_id where element_id = :element_id"
    } else {
	# this element has its own config.  Create it anew...
	ad_require_permission $config_id write
	db_dml delete_parameters \
	    "delete from portal_element_parameters where config_id = :config_id"
    }

    # go through the ns_set, adding parameters that occur in the default only.

    for {set i 0} {$i < [ns_set size $form]} {incr i} {
	set key [ns_set key $form $i]
	# check for that key.  If it exists, remove it.  If not, continue.

	set default_index [lsearch -exact $default_keys $key]
	if { $default_index < 0} {
	    continue
	} else {
	    set default_keys [lreplace $default_keys $default_index $default_index {}]
	}

	set value [ns_set value $form $i]
	db_dml insert_parameter \
	    "insert into portal_element_parameters
               (parameter_id, config_id, key, value)
             values
               (acs_object_id_seq.nextval, :config_id, :key, :value)"
    }
}

ns_returnredirect "index"
