# www/admin/element-delete.tcl

ad_page_contract {
    Remove an element, or make it unavailable.

    @author Ian Baker (ibaker@arsdigita.com)
    @creation-date 12/14/2000
    @cvs-id $Id: element-delete.tcl,v 1.1 2002/07/09 17:35:10 rmello Exp $
} {
    element_id:naturalnum,notnull
}

set user_id [ad_conn user_id]
set package_id [ad_conn package_id]
set default_portal_id [portal_info default_portal_id]

ad_require_permission $package_id admin

db_transaction {
    db_0or1row determine_ownership {
	select
	decode(m.element_id, e.element_id, 1, 0) as local_p,
	decode(acs_permission.permission_p(:element_id, :user_id, 'delete'), 't', 1, 'f', 0) as element_delete_p
	from portal_available_element_map m, portal_elements e
	where
	m.element_id = e.element_id and
	m.element_id = :element_id and
	m.portal_id = :default_portal_id
    }

    # if it's a locally-owned element (one that's controlled by this
    # package's default portal), delete it.  otherwise, just make it
    # unavailable to this package's default portal.
    if { $local_p } {
	if { ! $element_delete_p } {
	    ad_return_complaint "You don't have permission to delete this element."
	    return
	}
	
	db_dml delete_element \
	    "begin
               portal_element.delete(:element_id);
             end;"
    } else {
	db_dml remove_availability \
	    "delete from portal_available_element_map where element_id = :element_id and portal_id = :default_portal_id"
    }
}

ns_returnredirect "index.tcl"

