# www/admin/portal-ae-2.tcl

ad_page_contract {
    Generate a page with the same layout as the portal, for editing.

    @author Ian Baker (ibaker@arsdigita.com)
    @creation-date 12/6/2000
    @cvs-id $Id: portal-ae-2.tcl,v 1.1 2002/07/09 17:35:10 rmello Exp $
} {
    name:nohtml,notnull
    template_id:naturalnum,notnull,permission(read),optional
    {edit_default_p:naturalnum,notnull 0}
    portal_id:naturalnum,notnull
    a_template_ids:naturalnum,notnull,multiple,optional
}

set package_id [ad_conn package_id]
set user_id [ad_conn user_id]
set master_template [ad_parameter master_template]

# check perms.
if { ! [ ad_permission_p $package_id write ] } {
    ad_return_complaint 1 "Inadequate permissions."
    return
}

if {$edit_default_p && ![ad_permission_p $package_id admin] } {
    ad_return_complaint 1 "You don't have permission to administer this portal."
    return
}

set context_bar "\"Select Elements\""

# what's the default portal for this package (if any)
set default_portal_id [portal_info default_portal_id]

if { ! $edit_default_p && ! $default_portal_id } {
    ad_return_complaint 1 "There is no default portal for this package.  You need to create one of those first."
}

if { ![info exists a_template_ids] && [info exists template_id] } {
    # they didn't specify any templates; make the current one
    # available, atleast.
    lappend a_template_ids $template_id
}

if { ! $edit_default_p && [info exists template_id] } {
    # can we use this template?
    db_0or1row check_tmpl_availability \
	"select 1 as available_p
         from portal_available_template_map
         where
           portal_id = portal.parent(:default_portal_id) and
           template_id = :template_id"

    if { ! [info exists available_p] } {
	ad_return_complaint 1 "The template you've specified can't be used within this portal.  Please choose another."
	return
    }
}


db_transaction {
    # does the portal exist?
    db_0or1row determine_portal_existence \
	"select
           1 as portal_exists_p,
           decode(portal.root_p(portal_id), 't', 1, 'f', 0) as portal_root_p,
           decode(acs_permission.permission_p(portal_id, :user_id, 'write'), 't', 1, 'f', 0) as portal_write_p
         from portals
         where portal_id = :portal_id"

    if { [info exists portal_exists_p] } {
	# the portal exists.  Can this user edit it?
	if { ! $portal_write_p } {
	    ad_return_complaint 1 "You don't have permission to edit this portal."
	    db_abort_transaction
	    return
	}
	
	# which vars should be set?
	# (yes, I know, dynamic SQL is bad.  tell me how to do it otherwise, k?)
	foreach var [ list name template_id ] {
	    if { [info exists $var] } {
		lappend things_to_set "$var = :$var"
	    }
	}
	
	db_dml update_portal \
	    "update portals set [join $things_to_set ","] where portal_id = :portal_id"
	
	# only do this if we're supposed to, if there's permission, and if this
	# portal isn't inheriting these values from somewhere else.
	if { $edit_default_p && [ad_permission_p $package_id admin] && $portal_root_p } {
	    db_dml delete_available "delete from portal_available_template_map where portal_id = :portal_id"
	    foreach tid $a_template_ids {
		db_dml insert_available \
		    "insert into portal_available_template_map (portal_id,template_id) values (:portal_id, :tid)"
	    }
	}
    } else {
	# portal doesn't exist yet.  Create it.
	if { $edit_default_p && [ad_permission_p $package_id admin] } {
	    # default portal.
	    db_exec_plsql insert_def_portal \
		"declare
                   pid portals.portal_id%TYPE;
                 begin
                   pid := portal.new (
                     portal_id => :portal_id,
                     name => :name,
                     template_id => :template_id,
                     package_id => :package_id
                   );
                 end;"

	    foreach tid $a_template_ids {
		db_dml insert_available \
		    "insert into portal_available_template_map (portal_id,template_id) values (:portal_id, :tid)"
	    } 
	} else {
	    # create a user portal.  portal.new handles copying of the default.

	    # undefined name?
	    if { ! [info exists name] } {
		set name {}
	    }

	    # insert the portal and grant permission on it.

	    db_exec_plsql insert_user_portal {
		declare
		pid portals.portal_id%TYPE;
		begin
		pid := portal.new ( portal_id => :portal_id,
				    name => :name,
				    owner_id => :user_id,
				    parent_portal_id => :default_portal_id );

		acs_permission.grant_permission ( object_id => pid,
						  grantee_id => :user_id,
						  privilege => 'read' );
		
		acs_permission.grant_permission ( object_id => pid,
						  grantee_id => :user_id,
						  privilege => 'write' );

		acs_permission.grant_permission ( object_id => pid,
						  grantee_id => :user_id,
						  privilege => 'admin' );
		end;
	    }
	}
    }
}

ns_returnredirect "element-layout?[export_url_vars portal_id]"
