# www/admin/portal-ae.tcl

ad_page_contract {
    Add/edit a portal.

    @author Ian Baker (ibaker@arsdigita.com)
    @creation-date 12/6/2000
    @cvs-id $Id: portal-ae.tcl,v 1.1 2002/07/09 17:35:10 rmello Exp $
} {
    {edit_default_p:naturalnum,notnull,optional 0}
    portal_id:naturalnum,notnull,optional,object_write
}

set package_id [ad_conn package_id]
set user_id [ad_conn user_id]
set master_template [ad_parameter master_template]
set default_portal_id [portal_info default_portal_id]
set create_p 0

# one needs create permission on the package (at least) to use this at all.
# is this right?  I don't think so...
if { ! [ ad_permission_p $package_id create ] } {
    ad_return_complaint 1 "Inadequate permissions."
    return
}

# was a specific portal specified?  If not, find out which one to
# edit, or if a new one should be created.
if { ! [info exists portal_id] } {
    if { $edit_default_p } {
	set owner_where "owner_id is null"
    } else {
	set owner_where "owner_id = :user_id"
    }
    
    db_0or1row select_portal_id \
	"select portal_id from portals where package_id = :package_id and $owner_where"

    if { ! [info exists portal_id] } {

	if { ! [ ad_permission_p $package_id create ] } {
	    ad_return_complaint 1 "You don't have permission to create a personal portal.."
	    return
	}

	set create_p 1
	set portal_id [db_nextval acs_object_id_seq]
    } elseif { ! [ad_permission_p $portal_id write] } {
	ad_return_complaint 1 "You don't have permission to edit this portal."
    }
} else {
    db_1row select_owner_id "select owner_id from portals where package_id = :package_id and portal_id = :portal_id"

    # is this the default portal we're talking about here?
    if { [empty_string_p $owner_id] } {
	set edit_default_p 1
    }
}


# get the correct list of layout templates.
if { $edit_default_p } {
    if {! [ad_permission_p $package_id admin] } {
	ad_return_complaint 1 "You don't have permission to administer this portal."
	return
    }

    set layout_sql_name "get_layouts_for_default"
} else {
    set layout_sql_name "get_layouts_for_user"
}

set layout_count 0

template::multirow create layouts template_id name description filename resource_dir checked
db_foreach $layout_sql_name "" -column_array le {
    if { ! [ regexp {^/} $le(resource_dir)] } {
	set le(resource_dir) "/packages/portal/www/$le(resource_dir)"
    }

    # this is evil and broken.  I should be able to pass it a list, straight from db_list.
    template::multirow append layouts $le(template_id) $le(name) $le(description) $le(filename) $le(resource_dir) $le(checked)
    incr layout_count
}

if { $layout_count > 1 } {
    set title "Edit Portal - Name/Layout"
} else {
    set title "Edit Portal - Name"
}

set context_bar "\"Edit Portal\""

# now, get the correct data and return it.
if {$create_p && $edit_default_p} {
    # there's no data at all.  Just return the template.
    set name {}
    set template_id {}
    ad_return_template
    return
} elseif {$create_p} {
    # get the default portal.
    db_0or1row get_default_portal \
	"select
           name,
           template_id
         from portals
         where portal_id = :default_portal_id"
    
    if { [info exists name ] } {
	ad_return_template
	return
    } else {
	# there's no default.  Redirect here for admin.
	ns_returnredirect "[ad_conn url]?edit_default_p=1"
	return
    }
} else {
    # the portal we're editing exists.  Return it.
    db_1row get_portal "select
       portal_id,
       name,
       template_id
     from portals
     where package_id = :package_id and $owner_where"
    ad_return_template
}
